@@ -1,3 +1,4 @@
+# frozen_string_literal: true
 require 'tmpdir'
 require 'fileutils'
 require 'open-uri'
@@ -12,6 +13,9 @@ module YARD
         super
         @list_all = false
         @use_git = false
+        @compact = false
+        @modified = true
+        @verifier = Verifier.new
         @old_git_commit = nil
         @old_path = Dir.pwd
         log.show_backtraces = true
@@ -25,46 +29,78 @@ module YARD
         registry = optparse(*args).map do |gemfile|
           if @use_git
             load_git_commit(gemfile)
-            Registry.all.map {|o| o.path }
+            all_objects
+          elsif load_gem_data(gemfile)
+            log.info "Found #{gemfile}"
+            all_objects
           else
-            if load_gem_data(gemfile)
-              log.info "Found #{gemfile}"
-              Registry.all.map {|o| o.path }
-            else
-              log.error "Cannot find gem #{gemfile}"
-              nil
-            end
+            log.error "Cannot find gem #{gemfile}"
+            nil
           end
         end.compact
 
         return if registry.size != 2
 
-        [   ["Added objects", registry[1] - registry[0]],
-            ["Removed objects", registry[0] - registry[1]]].each do |name, objects|
+        first_object = nil
+        [["Added objects", "A", added_objects(*registry)],
+            ["Modified objects", "M", modified_objects(*registry)],
+            ["Removed objects", "D", removed_objects(*registry)]].each do |name, short, objects|
+          next if short == "M" && @modified == false
           next if objects.empty?
           last_object = nil
           all_objects_notice = false
-          puts name + ":"
-          objects.sort.each do |object|
-            if !@list_all && last_object && object =~ /#{Regexp.quote last_object}(::|\.|#)/
-              print " (...)" unless all_objects_notice
+          log.puts name + ":" unless @compact
+          objects.sort_by(&:path).each do |object|
+            if !@list_all && last_object && object.parent == last_object
+              log.print " (...)" unless all_objects_notice
               all_objects_notice = true
               next
+            elsif @compact
+              log.puts if first_object
             else
-              puts
+              log.puts
             end
             all_objects_notice = false
-            print "  " + object
+            log.print "" + (@compact ? "#{short} " : "  ") +
+                      object.path + " (#{object.file}:#{object.line})"
             last_object = object
+            first_object = true
+          end
+          unless @compact
+            log.puts; log.puts
           end
-          puts
-          puts
         end
+        log.puts if @compact
       end
 
       private
 
+      def all_objects
+        return Registry.all if @verifier.expressions.empty?
+        @verifier.run(Registry.all)
+      end
+
+      def added_objects(registry1, registry2)
+        registry2.reject {|o| registry1.find {|o2| o2.path == o.path } }
+      end
+
+      def modified_objects(registry1, registry2)
+        registry1.select do |obj|
+          case obj
+          when CodeObjects::MethodObject
+            registry2.find {|o| obj == o && o.source != obj.source }
+          when CodeObjects::ConstantObject
+            registry2.find {|o| obj == o && o.value != obj.value }
+          end
+        end.compact
+      end
+
+      def removed_objects(registry1, registry2)
+        registry1.reject {|o| registry2.find {|o2| o2.path == o.path } }
+      end
+
       def load_git_commit(commit)
+        Registry.clear
         commit_path = 'git_commit' + commit.gsub(/\W/, '_')
         tmpdir = File.join(Dir.tmpdir, commit_path)
         log.info "Expanding #{commit} to #{tmpdir}..."
@@ -74,6 +110,7 @@ module YARD
         Dir.chdir(tmpdir)
         log.info("git says: " + `git reset --hard #{commit}`.chomp)
         generate_yardoc(tmpdir)
+      ensure
         Dir.chdir(@old_path)
         cleanup(commit_path)
       end
@@ -85,30 +122,29 @@ module YARD
         # First check for argument as .yardoc file
         [File.join(gemfile, '.yardoc'), gemfile].each do |yardoc|
           log.info "Searching for .yardoc db at #{yardoc}"
-          if File.directory?(yardoc)
-            Registry.load_yardoc(yardoc)
-            Registry.load_all
-            return true
-          end
+          next unless File.directory?(yardoc)
+          Registry.load_yardoc(yardoc)
+          Registry.load_all
+          return true
         end
 
         # Next check installed RubyGems
         gemfile_without_ext = gemfile.sub(/\.gem$/, '')
         log.info "Searching for installed gem #{gemfile_without_ext}"
-        Gem.source_index.find_name('').find do |spec|
-          if spec.full_name == gemfile_without_ext
-            if yardoc = Registry.yardoc_file_for_gem(spec.name, "= #{spec.version}")
-              Registry.load_yardoc(yardoc)
-              Registry.load_all
-            else
-              log.enter_level(Logger::ERROR) do
-                olddir = Dir.pwd
-                Gems.run(spec.name, spec.version.to_s)
-                Dir.chdir(olddir)
-              end
+        YARD::GemIndex.find_all_by_name('').find do |spec|
+          next unless spec.full_name == gemfile_without_ext
+          yardoc = Registry.yardoc_file_for_gem(spec.name, "= #{spec.version}")
+          if yardoc
+            Registry.load_yardoc(yardoc)
+            Registry.load_all
+          else
+            log.enter_level(Logger::ERROR) do
+              olddir = Dir.pwd
+              Gems.run(spec.name, spec.version.to_s)
+              Dir.chdir(olddir)
             end
-            return true
           end
+          return true
         end
 
         # Look for local .gem file
@@ -128,6 +164,7 @@ module YARD
           open(url) {|io| expand_and_parse(gemfile, io) }
           return true
         rescue OpenURI::HTTPError
+          nil # noop
         end
         false
       end
@@ -139,21 +176,46 @@ module YARD
       end
 
       def generate_yardoc(dir)
-        olddir = Dir.pwd
-        Dir.chdir(dir)
-        log.enter_level(Logger::ERROR) { Yardoc.run('-n', '--no-save') }
-        Dir.chdir(olddir)
+        Dir.chdir(dir) do
+          log.enter_level(Logger::ERROR) { Yardoc.run('-n', '--no-save') }
+        end
       end
 
       def expand_gem(gemfile, io)
         tmpdir = File.join(Dir.tmpdir, gemfile)
-        log.info "Expanding #{gemfile} to #{tmpdir}..."
         FileUtils.mkdir_p(tmpdir)
-        Gem::Package.open(io) do |pkg|
-          pkg.each do |entry|
-            pkg.extract_entry(tmpdir, entry)
+        log.info "Expanding #{gemfile} to #{tmpdir}..."
+
+        if Gem::VERSION >= '2.0.0'
+          require 'rubygems/package/tar_reader'
+          reader = Gem::Package::TarReader.new(io)
+          reader.each do |pkg|
+            next unless pkg.full_name == 'data.tar.gz'
+            Zlib::GzipReader.wrap(pkg) do |gzio|
+              tar = Gem::Package::TarReader.new(gzio)
+              tar.each do |entry|
+                file = File.join(tmpdir, entry.full_name)
+                FileUtils.mkdir_p(File.dirname(file))
+                File.open(file, 'wb') do |out|
+                  out.write(entry.read)
+                  begin
+                    out.fsync
+                  rescue NotImplementedError
+                    nil # noop
+                  end
+                end
+              end
+            end
+            break
+          end
+        else
+          Gem::Package.open(io) do |pkg|
+            pkg.each do |entry|
+              pkg.extract_entry(tmpdir, entry)
+            end
           end
         end
+
         tmpdir
       end
 
@@ -184,13 +246,20 @@ module YARD
         opts.on('-a', '--all', 'List all objects, even if they are inside added/removed module/class') do
           @list_all = true
         end
+        opts.on('--compact', 'Show compact results') { @compact = true }
         opts.on('--git', 'Compare versions from two git commit/branches') do
           @use_git = true
         end
+        opts.on('--query QUERY', 'Only diff filtered objects') do |query|
+          @verifier.add_expressions(query)
+        end
+        opts.on('--no-modified', 'Ignore modified objects') do
+          @modified = false
+        end
         common_options(opts)
         parse_options(opts, args)
         unless args.size == 2
-          puts opts.banner
+          log.puts opts.banner
           exit(0)
         end
 
@@ -198,4 +267,4 @@ module YARD
       end
     end
   end
-end
\ No newline at end of file
+end
