@@ -1,3 +1,4 @@
+# frozen_string_literal: true
 module YARD
   module CLI
     # @since 0.6.0
@@ -45,8 +46,9 @@ module YARD
 
       def add_gems(gems)
         0.step(gems.size - 1, 2) do |index|
-          gem, ver_require = gems[index], gems[index + 1] || ">= 0"
-          specs = Gem.source_index.find_name(gem, ver_require)
+          gem = gems[index]
+          ver_require = gems[index + 1] || ">= 0"
+          specs = YARD::GemIndex.find_all_by_name(gem, ver_require)
           if specs.empty?
             log.warn "#{gem} #{ver_require} could not be found in RubyGems index"
           else
@@ -71,13 +73,12 @@ module YARD
         parse_options(opts, args)
         add_gems(args)
 
-
         if !args.empty? && @gems.empty?
           log.error "No specified gems could be found for command"
         elsif @gems.empty?
-          @gems += Gem.source_index.find_name('') if @gems.empty?
+          @gems += YARD::GemIndex.find_all_by_name('') if @gems.empty?
         end
       end
     end
   end
-end
\ No newline at end of file
+end
