@@ -1,3 +1,4 @@
+# frozen_string_literal: true
 require 'digest/sha1'
 require 'fileutils'
 
@@ -45,6 +46,9 @@ module YARD
       #   The +object+ key is not used so that a file may be rendered in the context
       #   of an object's namespace (for generating links).
       attr_accessor :file
+
+      # @return [String] the current locale
+      attr_accessor :locale
     end
 
     # Yardoc is the default YARD CLI command (+yard doc+ and historic +yardoc+
@@ -216,7 +220,10 @@ module YARD
         @has_markup = false
 
         if defined?(::Encoding) && ::Encoding.respond_to?(:default_external=)
-          ::Encoding.default_external, ::Encoding.default_internal = 'utf-8', 'utf-8'
+          utf8 = ::Encoding.find('utf-8')
+
+          ::Encoding.default_external = utf8 unless ::Encoding.default_external == utf8
+          ::Encoding.default_internal = utf8 unless ::Encoding.default_internal == utf8
         end
       end
 
@@ -232,7 +239,7 @@ module YARD
       # @return [void]
       def run(*args)
         log.show_progress = true
-        if args.size == 0 || !args.first.nil?
+        if args.empty? || !args.first.nil?
           # fail early if arguments are not valid
           return unless parse_arguments(*args)
         end
@@ -242,8 +249,15 @@ module YARD
           Registry.load
           checksums = Registry.checksums.dup
         end
-        YARD.parse(files, excluded)
-        Registry.save(use_cache) if save_yardoc
+
+        if save_yardoc
+          Registry.lock_for_writing do
+            YARD.parse(files, excluded)
+            Registry.save(use_cache)
+          end
+        else
+          YARD.parse(files, excluded)
+        end
 
         if generate
           run_generate(checksums)
@@ -272,9 +286,9 @@ module YARD
         super(*args)
 
         # Last minute modifications
-        self.files = ['{lib,app}/**/*.rb', 'ext/**/*.c'] if self.files.empty?
-        self.files.delete_if {|x| x =~ /\A\s*\Z/ } # remove empty ones
-        readme = Dir.glob('README*').first
+        self.files = Parser::SourceParser::DEFAULT_PATH_GLOB if files.empty?
+        files.delete_if {|x| x =~ /\A\s*\Z/ } # remove empty ones
+        readme = Dir.glob('README{,*[^~]}').first
         readme ||= Dir.glob(files.first).first if options.onefile
         options.readme ||= CodeObjects::ExtraFileObject.new(readme) if readme
         options.files.unshift(options.readme).uniq! if options.readme
@@ -325,7 +339,7 @@ module YARD
         Registry.load_all if use_cache
         objects = run_verifier(all_objects).reject do |object|
           serialized = !options.serializer || options.serializer.exists?(object)
-          if checksums && serialized && !object.files.any? {|f, line| changed_files.include?(f) }
+          if checksums && serialized && !object.files.any? {|f, _line| changed_files.include?(f) }
             true
           else
             log.debug "Re-generating object #{object.path}..."
@@ -340,7 +354,8 @@ module YARD
       #
       # @return (see YARD::Templates::Helpers::MarkupHelper#load_markup_provider)
       def verify_markup_options
-        result, lvl = false, has_markup ? log.level : Logger::FATAL
+        result = false
+        lvl = has_markup ? log.level : Logger::FATAL
         obj = Struct.new(:options).new(options)
         obj.extend(Templates::Helpers::MarkupHelper)
         options.files.each do |file|
@@ -351,8 +366,8 @@ module YARD
         options.markup = :rdoc unless has_markup
         log.enter_level(lvl) { result = obj.load_markup_provider }
         if !result && !has_markup
-          log.warn "Could not load default RDoc formatter, " +
-            "ignoring any markup (install RDoc to get default formatting)."
+          log.warn "Could not load default RDoc formatter, " \
+                   "ignoring any markup (install RDoc to get default formatting)."
           options.markup = :none
           true
         else
@@ -439,15 +454,13 @@ module YARD
         no_api = true if apis.delete('')
         exprs = []
 
-        if apis.size > 0
-          exprs << "#{apis.uniq.inspect}.include?(@api.text)"
-        end
+        exprs << "#{apis.uniq.inspect}.include?(@api.text)" unless apis.empty?
 
-        if hidden_apis.size > 0
+        unless hidden_apis.empty?
           exprs << "!#{hidden_apis.uniq.inspect}.include?(@api.text)"
         end
 
-        exprs = exprs.size > 0 ? [exprs.join(' && ')] : []
+        exprs = !exprs.empty? ? [exprs.join(' && ')] : []
         exprs << "!@api" if no_api
 
         expr = exprs.join(' || ')
@@ -540,7 +553,7 @@ module YARD
         end
 
         opts.on('--exclude REGEXP', 'Ignores a file if it matches path match (regexp)') do |path|
-          self.excluded << path
+          excluded << path
         end
       end
 
@@ -553,7 +566,7 @@ module YARD
           options.onefile = true
         end
 
-        opts.on('--list', 'List objects to standard out (implies -n)') do |format|
+        opts.on('--list', 'List objects to standard out (implies -n)') do |_format|
           self.generate = false
           self.list = true
         end
@@ -635,7 +648,7 @@ module YARD
 
         opts.on('--asset FROM[:TO]', 'A file or directory to copy over to output ',
                                      '  directory after generating') do |asset|
-          re = /^(?:\.\.\/|\/)/
+          re = %r{^(?:\.\./|/)}
           from, to = *asset.split(':').map {|f| File.cleanpath(f) }
           to ||= from
           if from =~ re || to =~ re
@@ -667,7 +680,8 @@ module YARD
                                  '  (default is system locale)') do |encoding|
           begin
             if defined?(Encoding) && Encoding.respond_to?(:default_external=)
-              Encoding.default_external, Encoding.default_internal = encoding, encoding
+              Encoding.default_external = encoding
+              Encoding.default_internal = encoding
             end
           rescue ArgumentError => e
             raise OptionParser::InvalidOption, e
@@ -696,6 +710,10 @@ module YARD
           self.statistics = false
         end
 
+        opts.on('--no-progress', 'Don\'t show progress bar') do
+          log.show_progress = false
+        end
+
         opts.on('--locale LOCALE',
                 'The locale for generated documentation.',
                 '  (defaults to en)') do |locale|
