@@ -1,3 +1,4 @@
+# frozen_string_literal: true
 require 'rbconfig'
 
 module YARD
@@ -6,12 +7,12 @@ module YARD
     class YRI < Command
       # The location in {YARD::CONFIG_DIR} where the YRI cache file is loaded
       # from.
-      CACHE_FILE = File.expand_path('~/.yard/yri_cache')
+      CACHE_FILE = File.expand_path(File.join(YARD::Config::CONFIG_DIR, 'yri_cache'))
 
       # A file containing all paths, delimited by newlines, to search for
       # yardoc databases.
       # @since 0.5.1
-      SEARCH_PATHS_FILE = File.expand_path('~/.yard/yri_search_paths')
+      SEARCH_PATHS_FILE = File.expand_path(File.join(YARD::Config::CONFIG_DIR, 'yri_search_paths'))
 
       # Default search paths that should be loaded dynamically into YRI. These paths
       # take precedence over all other paths ({SEARCH_PATHS_FILE} and RubyGems
@@ -57,12 +58,15 @@ module YARD
 
         if @name.nil? || @name.strip.empty?
           print_usage
-          exit(1)
-        elsif object = find_object(@name)
+          return exit(1)
+        end
+
+        object = find_object(@name)
+        if object
           print_object(object)
         else
           STDERR.puts "No documentation for `#{@name}'"
-          exit(1)
+          return exit(1)
         end
       end
 
@@ -72,8 +76,8 @@ module YARD
       # @return [void]
       # @since 0.5.6
       def print_usage
-        puts "Usage: yri [options] <Path to object>"
-        puts "See yri --help for more options."
+        log.puts "Usage: yri [options] <Path to object>"
+        log.puts "See yri --help for more options."
       end
 
       # Caches the .yardoc file where an object can be found in the {CACHE_FILE}
@@ -112,18 +116,16 @@ module YARD
 
         # Try to load it from in memory cache
         log.debug "Searching for #{name} in memory"
-        if obj = try_load_object(name, nil)
-          return obj
-        end
+        obj = try_load_object(name, nil)
+        return obj if obj
 
         log.debug "Searching for #{name} in search paths"
         @search_paths.each do |path|
           next unless File.exist?(path)
           log.debug "Searching for #{name} in #{path}..."
           Registry.load(path)
-          if obj = try_load_object(name, path)
-            return obj
-          end
+          obj = try_load_object(name, path)
+          return obj if obj
         end
         nil
       end
@@ -139,12 +141,9 @@ module YARD
       #   No caching is done if this is nil.
       # @return [void]
       def try_load_object(name, cache_path)
-        if obj = Registry.at(name)
-          if cache_path
-            cache_object(name, cache_path)
-          end
-          return obj
-        end
+        obj = Registry.at(name)
+        cache_object(name, cache_path) if obj && cache_path
+        obj
       end
 
       # Loads {CACHE_FILE}
@@ -162,26 +161,28 @@ module YARD
       def add_gem_paths
         require 'rubygems'
         gem_paths = []
-        Gem.source_index.find_name('').each do |spec|
-          if yfile = Registry.yardoc_file_for_gem(spec.name)
-            if spec.name =~ /^yard-doc-/
-              gem_paths.unshift(yfile)
-            else
-              gem_paths.push(yfile)
-            end
+        YARD::GemIndex.find_all_by_name('').each do |spec|
+          yfile = Registry.yardoc_file_for_gem(spec.name)
+          next if yfile.nil?
+
+          if spec.name =~ /^yard-doc-/
+            gem_paths.unshift(yfile)
+          else
+            gem_paths.push(yfile)
           end
         end
         @search_paths += gem_paths
       rescue LoadError
+        nil # noop
       end
 
       # Adds paths in {SEARCH_PATHS_FILE}
       # @since 0.5.1
       def add_default_paths
-        @search_paths.push(*DEFAULT_SEARCH_PATHS)
+        @search_paths.concat(DEFAULT_SEARCH_PATHS)
         return unless File.file?(SEARCH_PATHS_FILE)
-        paths = File.readlines(SEARCH_PATHS_FILE).map {|l| l.strip }
-        @search_paths.push(*paths)
+        paths = File.readlines(SEARCH_PATHS_FILE).map(&:strip)
+        @search_paths.concat(paths)
       end
 
       # Parses commandline options.
