@@ -1,3 +1,4 @@
+# frozen_string_literal: true
 module YARD
   # This class maintains all system-wide configuration for YARD and handles
   # the loading of plugins. To access options call {options}, and to load
@@ -123,8 +124,9 @@ module YARD
       add_ignored_plugins_file
       translate_plugin_names
       load_plugins
-    rescue
+    rescue => e
       log.error "Invalid configuration file, using default options."
+      log.backtrace(e)
       options.update(DEFAULT_CONFIG_OPTIONS)
     end
 
@@ -163,20 +165,18 @@ module YARD
       load_plugin_failed(name, e)
     end
 
-    private
-
     # Load gem plugins if :load_plugins is true
     def self.load_gem_plugins
       return true unless options[:load_plugins]
       require 'rubygems'
       result = true
-      Gem.source_index.find_name('').each do |gem|
+      YARD::GemIndex.find_all_by_name('').each do |gem|
         begin
           next true unless gem.name =~ YARD_PLUGIN_PREFIX
           load_plugin(gem.name)
         rescue Gem::LoadError => e
           tmp = load_plugin_failed(gem.name, e)
-          result = tmp if !tmp
+          result = tmp unless tmp
         end
       end
       result
@@ -190,20 +190,20 @@ module YARD
       options[:autoload_plugins].each {|name| load_plugin(name) }
     end
 
-    # Load plugins from {#arguments}
+    # Load plugins from {arguments}
     def self.load_commandline_plugins
       with_yardopts do
         arguments.each_with_index do |arg, i|
           next unless arg == '--plugin'
-          load_plugin(arguments[i+1])
+          load_plugin(arguments[i + 1])
         end
       end
     end
 
-    # Check for command-line safe_mode switch in {#arguments}
+    # Check for command-line safe_mode switch in {arguments}
     def self.load_commandline_safemode
       with_yardopts do
-        arguments.each_with_index do |arg, i|
+        arguments.each_with_index do |arg, _i|
           options[:safe_mode] = true if arg == '--safe'
         end
       end
@@ -212,7 +212,7 @@ module YARD
     # Print a warning if the plugin failed to load
     # @return [false]
     def self.load_plugin_failed(name, exception)
-      log.warn "Error loading plugin '#{name}'"
+      log.error "Error loading plugin '#{name}'"
       log.backtrace(exception) if $DEBUG
       false
     end
@@ -246,13 +246,13 @@ module YARD
     # @param [String] name the plugin name
     # @return [String] the sanitized and normalized plugin name.
     def self.translate_plugin_name(name)
-      name = name.gsub('/', '') # Security sanitization
+      name = name.delete('/') # Security sanitization
       name = "yard-" + name unless name =~ YARD_PLUGIN_PREFIX
       name
     end
 
     # Temporarily loads .yardopts file into @yardopts
-    def self.with_yardopts(&block)
+    def self.with_yardopts
       yfile = CLI::Yardoc::DEFAULT_YARDOPTS_FILE
       @yardopts = File.file?(yfile) ? File.read_binary(yfile).shell_split : []
       result = yield
