@@ -1,3 +1,4 @@
+# frozen_string_literal: true
 require 'ostruct'
 
 module YARD
@@ -117,7 +118,7 @@ module YARD
       # Remove trailing/leading whitespace / newlines
       @text = text.gsub(/\A[\r\n\s]+|[\r\n\s]+\Z/, '')
       call_directives_after_parse
-      call_after_parse_callbacks
+      post_process
       self
     end
 
@@ -129,15 +130,17 @@ module YARD
     def parse_content(content)
       content = content.split(/\r?\n/) if content.is_a?(String)
       return '' if !content || content.empty?
-      docstring = ""
+      docstring = String.new("")
 
-      indent, last_indent = content.first[/^\s*/].length, 0
+      indent = content.first[/^\s*/].length
+      last_indent = 0
       orig_indent = 0
       directive = false
       last_line = ""
-      tag_name, tag_klass, tag_buf = nil, nil, []
+      tag_name = nil
+      tag_buf = []
 
-      (content+['']).each_with_index do |line, index|
+      (content + ['']).each_with_index do |line, index|
         indent = line[/^\s*/].length
         empty = (line =~ /^\s*$/ ? true : false)
         done = content.size == index
@@ -153,13 +156,17 @@ module YARD
           else
             create_tag(tag_name, buf)
           end
-          tag_name, tag_buf, directive = nil, [], false
+          tag_name = nil
+          tag_buf = []
+          directive = false
           orig_indent = 0
         end
 
         # Found a meta tag
         if line =~ META_MATCH
-          directive, tag_name, tag_buf = $1, $2, [($3 || '')]
+          directive = $1
+          tag_name = $2
+          tag_buf = [($3 || '')]
         elsif tag_name && indent >= orig_indent && !empty
           orig_indent = indent if orig_indent == 0
           # Extra data added to the tag on the next line
@@ -169,7 +176,8 @@ module YARD
           tag_buf << line.gsub(/^[ \t]{#{orig_indent}}/, '')
         elsif !tag_name
           # Regular docstring text
-          docstring << line << "\n"
+          docstring << line
+          docstring << "\n"
         end
 
         last_indent = indent
@@ -179,6 +187,17 @@ module YARD
       docstring
     end
 
+    # @!group Parser Callback Methods
+
+    # Call post processing callbacks on parser.
+    # This is called implicitly by parser. Use this when
+    # manually configuring a {Docstring} object.
+    #
+    # @return [void]
+    def post_process
+      call_after_parse_callbacks
+    end
+
     # @!group Tag Manipulation Methods
 
     # Creates a tag from the {Tags::DefaultFactory tag factory}.
@@ -197,11 +216,11 @@ module YARD
         @tags += [library.tag_create(tag_name, tag_buf)].flatten
       else
         log.warn "Unknown tag @#{tag_name}" +
-          (object ? " in file `#{object.file}` near line #{object.line}" : "")
+                 (object ? " in file `#{object.file}` near line #{object.line}" : "")
       end
     rescue Tags::TagFormatError
       log.warn "Invalid tag format for @#{tag_name}" +
-        (object ? " in file `#{object.file}` near line #{object.line}" : "")
+               (object ? " in file `#{object.file}` near line #{object.line}" : "")
     end
 
     # Creates a {Tags::RefTag}
@@ -220,12 +239,12 @@ module YARD
         end
       else
         log.warn "Unknown directive @!#{tag_name}" +
-          (object ? " in file `#{object.file}` near line #{object.line}" : "")
+                 (object ? " in file `#{object.file}` near line #{object.line}" : "")
         nil
       end
     rescue Tags::TagFormatError
       log.warn "Invalid directive format for @!#{tag_name}" +
-        (object ? " in file `#{object.file}` near line #{object.line}" : "")
+               (object ? " in file `#{object.file}` near line #{object.line}" : "")
       nil
     end
 
@@ -236,44 +255,6 @@ module YARD
       list.include?(tag_name)
     end
 
-    private
-
-    def namespace
-      if object
-        object.namespace
-      else
-        nil
-      end
-    end
-
-    def detect_reference(content)
-      if content =~ /\A\s*\(see (\S+)\s*\)(?:\s|$)/
-        path, extra = $1, $'
-        [CodeObjects::Proxy.new(namespace, path), extra]
-      else
-        [nil, content]
-      end
-    end
-
-    # @!group Parser Callback Methods
-
-    # Calls the {Directive#after_parse} callback on all the
-    # created directives.
-    def call_directives_after_parse
-      directives.each do |dir|
-        dir.after_parse
-      end
-    end
-
-    # Calls all {after_parse} callbacks
-    def call_after_parse_callbacks
-      self.class.after_parse_callbacks.each do |cb|
-        cb.call(self)
-      end
-    end
-
-    public
-
     # Creates a callback that is called after a docstring is successfully
     # parsed. Use this method to perform sanity checks on a docstring's
     # tag data, or add any extra tags automatically to a docstring.
@@ -284,7 +265,7 @@ module YARD
     # @yieldreturn [void]
     # @return [void]
     def self.after_parse(&block)
-      self.after_parse_callbacks << block
+      after_parse_callbacks << block
     end
 
     # @return [Array<Proc>] the {after_parse} callback proc objects
@@ -296,26 +277,69 @@ module YARD
     after_parse do |parser|
       next unless parser.object
       next unless parser.object.is_a?(CodeObjects::MethodObject)
-      next if parser.object.parameters.empty? # method has no params or
-                                                # YARD couldn't detect any.
-                                                # but don't warn user (?)
+      next if parser.object.is_alias?
       names = parser.object.parameters.map {|l| l.first.gsub(/\W/, '') }
       seen_names = []
-      infile_info = "\n    in file `#{parser.object.file}' " +
+      infile_info = "\n    in file `#{parser.object.file}' " \
                     "near line #{parser.object.line}"
       parser.tags.each do |tag|
         next if tag.is_a?(Tags::RefTagList) # we don't handle this yet
         next unless tag.tag_name == "param"
         if seen_names.include?(tag.name)
-          log.warn "@param tag has duplicate parameter name: " +
-            "#{tag.name} #{infile_info}"
+          log.warn "@param tag has duplicate parameter name: " \
+                   "#{tag.name} #{infile_info}"
         elsif names.include?(tag.name)
           seen_names << tag.name
         else
-          log.warn "@param tag has unknown parameter name: " +
-            "#{tag.name} #{infile_info}"
+          log.warn "@param tag has unknown parameter name: " \
+                   "#{tag.name} #{infile_info}"
         end
       end
     end
+
+    private
+
+    def namespace
+      object && object.namespace
+    end
+
+    def detect_reference(content)
+      if content =~ /\A\s*\(see (\S+)\s*\)(?:\s|$)/
+        path = $1
+        extra = $'
+        [CodeObjects::Proxy.new(namespace, path), extra]
+      else
+        [nil, content]
+      end
+    end
+
+    # @!group Parser Callback Methods
+
+    # Calls the {Tags::Directive#after_parse} callback on all the
+    # created directives.
+    def call_directives_after_parse
+      directives.each(&:after_parse)
+    end
+
+    # Calls all {after_parse} callbacks
+    def call_after_parse_callbacks
+      self.class.after_parse_callbacks.each do |cb|
+        cb.call(self)
+      end
+    end
+
+    # Define a callback to check that @see tags do not use {}.
+    after_parse do |parser|
+      next unless parser.object
+
+      parser.tags.each_with_index do |tag, i|
+        next unless tag.tag_name == "see"
+        next unless "#{tag.name}#{tag.text}" =~ /\A\{.*\}\Z/
+        infile_info = "\n    in file `#{parser.object.file}' " \
+                      "near line #{parser.object.line}"
+        log.warn "@see tag (##{i + 1}) should not be wrapped in {} " \
+                 "(causes rendering issues): #{infile_info}"
+      end
+    end
   end
 end
