@@ -1,10 +1,12 @@
-begin require 'ripper'; rescue LoadError; end
+# frozen_string_literal: true
+begin require 'ripper'; rescue LoadError; nil end
 
 module YARD
   module Parser
     module Ruby
       # Ruby 1.9 parser
       # @!attribute [r] encoding_line
+      # @!attribute [r] frozen_string_line
       # @!attribute [r] shebang_line
       # @!attribute [r] enumerator
       class RubyParser < Parser::Base
@@ -17,13 +19,14 @@ module YARD
         def enumerator; @parser.enumerator end
         def shebang_line; @parser.shebang_line end
         def encoding_line; @parser.encoding_line end
+        def frozen_string_line; @parser.frozen_string_line end
       end
 
       # Internal parser class
       # @since 0.5.6
       class RipperParser < Ripper
         attr_reader :ast, :charno, :comments, :file, :tokens
-        attr_reader :shebang_line, :encoding_line
+        attr_reader :shebang_line, :encoding_line, :frozen_string_line
         alias root ast
 
         def initialize(source, filename, *args)
@@ -43,7 +46,10 @@ module YARD
           @charno = 0
           @shebang_line = nil
           @encoding_line = nil
+          @frozen_string_line = nil
           @file_encoding = nil
+          @newline = true
+          @percent_ary = nil
         end
 
         def parse
@@ -63,9 +69,8 @@ module YARD
           return nil unless defined?(::Encoding)
           return @file_encoding if @file_encoding
           return Encoding.default_internal unless @encoding_line
-          if match = @encoding_line.match(SourceParser::ENCODING_LINE)
-            @file_encoding = match.captures.last
-          end
+          match = @encoding_line.match(SourceParser::ENCODING_LINE)
+          @file_encoding = match.captures.last if match
         end
 
         private
@@ -97,6 +102,9 @@ module YARD
           :next => "next",
           :paren => :lparen,
           :qwords_literal => :qwords_beg,
+          :words_literal => :words_beg,
+          :qsymbols_literal => :qsymbols_beg,
+          :symbols_literal => :symbols_beg,
           :redo => "redo",
           :regexp_literal => :regexp_beg,
           :rescue => "rescue",
@@ -127,7 +135,7 @@ module YARD
 
         MAPPINGS.each do |k, v|
           if Array === v
-            v.each {|_v| (REV_MAPPINGS[_v] ||= []) << k }
+            v.each {|vv| (REV_MAPPINGS[vv] ||= []) << k }
           else
             (REV_MAPPINGS[v] ||= []) << k
           end
@@ -136,7 +144,7 @@ module YARD
         PARSER_EVENT_TABLE.each do |event, arity|
           node_class = AstNode.node_class_for(event)
 
-          if /_new\z/ =~ event.to_s and arity == 0
+          if /_new\z/ =~ event.to_s && arity == 0
             module_eval(<<-eof, __FILE__, __LINE__ + 1)
               def on_#{event}(*args)
                 #{node_class}.new(:list, args, :listchar => charno...charno, :listline => lineno..lineno)
@@ -150,7 +158,7 @@ module YARD
                 list
               end
             eof
-          elsif MAPPINGS.has_key?(event)
+          elsif MAPPINGS.key?(event)
             module_eval(<<-eof, __FILE__, __LINE__ + 1)
               begin; undef on_#{event}; rescue NameError; end
               def on_#{event}(*args)
@@ -177,8 +185,8 @@ module YARD
           eof
         end
 
-        REV_MAPPINGS.select {|k,v| k.is_a?(Symbol) }.each do |pair|
-          event, value = *pair
+        REV_MAPPINGS.select {|k, _v| k.is_a?(Symbol) }.each do |pair|
+          event = pair.first
           ast_token = AST_TOKENS.include?(event)
           module_eval(<<-eof, __FILE__, __LINE__ + 1)
             begin; undef on_#{event}; rescue NameError; end
@@ -194,7 +202,8 @@ module YARD
             begin; undef on_#{event}; rescue NameError; end
             def on_#{event}(tok)
               unless @last_ns_token == [:kw, "def"] ||
-                  (@tokens.last && @tokens.last[0] == :symbeg)
+                  (@tokens.last && @tokens.last[0] == :symbeg) ||
+                  (!@newline && %w(if while until unless).include?(tok))
                 (@map[tok] ||= []) << [lineno, charno]
               end
               visit_ns_token(:#{event}, tok, true)
@@ -202,24 +211,33 @@ module YARD
           eof
         end
 
-        [:sp, :nl, :ignored_nl].each do |event|
+        [:nl, :ignored_nl].each do |event|
           module_eval(<<-eof, __FILE__, __LINE__ + 1)
             begin; undef on_#{event}; rescue NameError; end
             def on_#{event}(tok)
               add_token(:#{event}, tok)
-              @charno += tok.length
+              @newline = true
+              @charno += tok ? tok.length : 1
             end
           eof
         end
 
+        undef on_sp
+
+        def on_sp(tok)
+          add_token(:sp, tok)
+          @charno += tok.length
+        end
+
         def visit_event(node)
           map = @map[MAPPINGS[node.type]]
           lstart, sstart = *(map ? map.pop : [lineno, @ns_charno - 1])
           node.source_range = Range.new(sstart, @ns_charno - 1)
           node.line_range = Range.new(lstart, lineno)
           if node.respond_to?(:block)
-            sr, lr = node.block.source_range, node.block.line_range
-            node.block.source_range = Range.new(sr.first, @tokens.last[2][1])
+            sr = node.block.source_range
+            lr = node.block.line_range
+            node.block.source_range = Range.new(sr.first, @tokens.last[2][1] - 1)
             node.block.line_range = Range.new(lr.first, @tokens.last[2][0])
           end
           node
@@ -239,13 +257,28 @@ module YARD
           @last_ns_token = [token, data]
           @charno += data.length
           @ns_charno = charno
+          @newline = [:semicolon, :comment, :kw, :op, :lparen, :lbrace].include?(token)
           if ast_token
-            AstNode.new(token, [data], :line => lineno..lineno, :char => ch..charno-1, :token => true)
+            AstNode.new(token, [data], :line => lineno..lineno, :char => ch..charno - 1, :token => true)
           end
         end
 
         def add_token(token, data)
-          if @tokens.last && @tokens.last[0] == :symbeg
+          if @percent_ary
+            if token == :words_sep && data !~ /\s\z/
+              rng = @percent_ary.source_range
+              rng = Range.new(rng.first, rng.last + data.length)
+              @percent_ary.source_range = rng
+              @tokens << [token, data, [lineno, charno]]
+              @percent_ary = nil
+            elsif token == :tstring_end && data =~ /\A\s/
+              rng = @percent_ary.source_range
+              rng = Range.new(rng.first, rng.last + data.length)
+              @percent_ary.source_range = rng
+              @tokens << [token, data, [lineno, charno]]
+              @percent_ary = nil
+            end
+          elsif @tokens.last && @tokens.last[0] == :symbeg
             @tokens[-1] = [:symbol, ":" + data, @tokens.last[2]]
           elsif @heredoc_state == :started
             @heredoc_tokens << [token, data, [lineno, charno]]
@@ -279,8 +312,6 @@ module YARD
         undef on_aref_field
         undef on_lbracket
         undef on_rbracket
-        undef on_qwords_new
-        undef on_qwords_add
         undef on_string_literal
         undef on_lambda
         undef on_unary
@@ -324,7 +355,7 @@ module YARD
         end
 
         def on_unary(op, val)
-          map = @map[op.to_s[0,1]]
+          map = @map[op.to_s[0, 1]]
           lstart, sstart = *(map ? map.pop : [lineno, @ns_charno - 1])
           node = AstNode.node_class_for(:unary).new(:unary, [op, val])
           node.source_range = Range.new(sstart, @ns_charno - 1)
@@ -390,7 +421,7 @@ module YARD
           ReferenceNode.new(:const_path_ref, args, :listline => lineno..lineno, :listchar => charno..charno)
         end
 
-        [:if_mod, :unless_mod, :while_mod].each do |kw|
+        [:if_mod, :unless_mod, :while_mod, :until_mod].each do |kw|
           node_class = AstNode.node_class_for(kw)
           module_eval(<<-eof, __FILE__, __LINE__ + 1)
             begin; undef on_#{kw}; rescue NameError; end
@@ -402,22 +433,29 @@ module YARD
           eof
         end
 
-        def on_qwords_new(*args)
-          node = LiteralNode.new(:qwords_literal, args)
-          if @map[:qwords_beg]
-            lstart, sstart = *@map[:qwords_beg].pop
-            node.source_range = Range.new(sstart, @ns_charno-1)
-            node.line_range = Range.new(lstart, lineno)
-          end
-          node
-        end
+        %w(symbols qsymbols words qwords).each do |kw|
+          module_eval(<<-eof, __FILE__, __LINE__ + 1)
+            begin; undef on_#{kw}_new; rescue NameError; end
+            def on_#{kw}_new(*args)
+              node = LiteralNode.new(:#{kw}_literal, args)
+              @percent_ary = node
+              if @map[:#{kw}_beg]
+                lstart, sstart = *@map[:#{kw}_beg].pop
+                node.source_range = Range.new(sstart, @ns_charno-1)
+                node.line_range = Range.new(lstart, lineno)
+              end
+              node
+            end
 
-        def on_qwords_add(list, item)
-          last = @source[@ns_charno,1] == "\n" ? @ns_charno - 1 : @ns_charno
-          list.source_range = (list.source_range.first..last)
-          list.line_range = (list.line_range.first..lineno)
-          list.push(item)
-          list
+            begin; undef on_#{kw}_add; rescue NameError; end
+            def on_#{kw}_add(list, item)
+              last = @source[@ns_charno,1] == "\n" ? @ns_charno - 1 : @ns_charno
+              list.source_range = (list.source_range.first..last)
+              list.line_range = (list.line_range.first..lineno)
+              list.push(item)
+              list
+            end
+          eof
         end
 
         def on_string_literal(*args)
@@ -452,21 +490,20 @@ module YARD
 
         def on_params(*args)
           args.map! do |arg|
-            if arg.class == Array
-              if arg.first.class == Array
-                arg.map! do |sub_arg|
-                  if sub_arg.class == Array
-                    AstNode.new(:default_arg, sub_arg, :listline => lineno..lineno, :listchar => charno..charno)
-                  else
-                    sub_arg
-                  end
-                end
+            next arg unless arg.class == Array
+
+            if arg.first.class == Array
+              arg.map! do |sub_arg|
+                next sub_arg unless sub_arg.class == Array
+                type = sub_arg[0].type == :label ?
+                  :named_arg : :unnamed_optional_arg
+                AstNode.new(type, sub_arg, :listline => lineno..lineno, :listchar => charno..charno)
               end
-              AstNode.new(:list, arg, :listline => lineno..lineno, :listchar => charno..charno)
-            else
-              arg
             end
+
+            AstNode.new(:list, arg, :listline => lineno..lineno, :listchar => charno..charno)
           end
+
           ParameterNode.new(:params, args, :listline => lineno..lineno, :listchar => charno..charno)
         end
 
@@ -475,18 +512,21 @@ module YARD
           ch = charno
           @charno += data.length
           @ns_charno = charno
-          AstNode.new(:label, [data[0...-1]], :line => lineno..lineno, :char => ch..charno-1, :token => true)
+          AstNode.new(:label, [data[0...-1]], :line => lineno..lineno, :char => ch..charno - 1, :token => true)
         end
 
         def on_comment(comment)
           not_comment = false
-          if @last_ns_token.nil? || @last_ns_token.size == 0
+          if @last_ns_token.nil? || @last_ns_token.empty?
             if comment =~ SourceParser::SHEBANG_LINE && !@encoding_line
               @shebang_line = comment
               not_comment = true
             elsif comment =~ SourceParser::ENCODING_LINE
               @encoding_line = comment
               not_comment = true
+            elsif comment =~ SourceParser::FROZEN_STRING_LINE
+              @frozen_string_line = comment
+              not_comment = true
             end
           end
 
@@ -497,13 +537,14 @@ module YARD
             return
           end
 
-          source_range = ch..(charno-1)
+          source_range = ch..(charno - 1)
           comment = comment.gsub(/^(\#+)\s{0,1}/, '').chomp
           append_comment = @comments[lineno - 1]
 
           hash_flag = $1 == '##' ? true : false
 
-          if append_comment && @comments_last_column && @comments_last_column == column
+          if append_comment && @comments_last_column &&
+             @comments_last_column == column && comment_starts_line?(ch)
             @comments.delete(lineno - 1)
             @comments_flags[lineno] = @comments_flags[lineno - 1]
             @comments_flags.delete(lineno - 1)
@@ -514,14 +555,14 @@ module YARD
 
           @comments[lineno] = comment
           @comments_range[lineno] = source_range
-          @comments_flags[lineno] = hash_flag if !append_comment
+          @comments_flags[lineno] = hash_flag unless append_comment
           @comments_last_column = column
         end
 
         def on_embdoc_beg(text)
           visit_ns_token(:embdoc_beg, text)
-          @embdoc_start = charno-text.length
-          @embdoc = ""
+          @embdoc_start = charno - text.length
+          @embdoc = String.new("")
         end
 
         def on_embdoc(text)
@@ -541,12 +582,28 @@ module YARD
         def on_parse_error(msg)
           raise ParserSyntaxError, "syntax error in `#{file}`:(#{lineno},#{column}): #{msg}"
         end
+        alias compile_error on_parse_error
+
+        def comment_starts_line?(charno)
+          (charno - 1).downto(0) do |i|
+            ch = @source[i]
+            break if ch == "\n"
+            return false if ch != " " && ch != "\t"
+          end
+          true
+        end
 
         def insert_comments
           root.traverse do |node|
             next if node.type == :comment || node.type == :list || node.parent.type != :list
+
+            # never attach comments to if/unless mod nodes
+            if node.type == :if_mod || node.type == :unless_mod
+              node = node.then_block
+            end
+
             # check upwards from line before node; check node's line at the end
-            ((node.line-1).downto(node.line-2).to_a + [node.line]).each  do |line|
+            ((node.line - 1).downto(node.line - 2).to_a + [node.line]).each do |line|
               comment = @comments[line]
               if comment && !comment.empty?
                 add_comment(line, node)
@@ -555,9 +612,7 @@ module YARD
             end
 
             @comments.keys.each do |line|
-              if node.line > line
-                add_comment(line, nil, node)
-              end
+              add_comment(line, nil, node) if node.line > line
             end
           end
 
@@ -565,21 +620,20 @@ module YARD
           root.traverse do |node|
             next if node.type == :list || node.parent.type != :list
             @comments.keys.each do |line|
-              if node.line_range.include?(line)
-                pick = nil
-                node.traverse do |subnode|
-                  next unless subnode.type == :list
-                  pick ||= subnode
-                  next unless subnode.line_range.include?(line)
-                  pick = subnode
-                end
-                add_comment(line, nil, pick, true) if pick
+              next unless node.line_range.include?(line)
+              pick = nil
+              node.traverse do |subnode|
+                next unless subnode.type == :list
+                pick ||= subnode
+                next unless subnode.line_range.include?(line)
+                pick = subnode
               end
+              add_comment(line, nil, pick, true) if pick
             end
-          end if @comments.size > 0
+          end unless @comments.empty?
 
           # insert all remaining comments
-          @comments.each do |line, comment|
+          @comments.each do |line, _comment|
             add_comment(line, nil, root, true)
           end
 
@@ -613,13 +667,16 @@ module YARD
         end
 
         def freeze_tree(node = nil)
-          node ||= root
-          node.children.each do |child|
-            child.parent = node
-            freeze_tree(child)
+          nodes = [node || root]
+          until nodes.empty?
+            p_node = nodes.shift
+            p_node.children.each do |child|
+              child.parent = p_node
+              nodes << child
+            end
           end
         end
       end if defined?(::Ripper)
     end
   end
-end
\ No newline at end of file
+end
